pro sph_coord_conv, long1, lat1, rtot_12, long2, lat2, inverse=inverse
;+
; NAME: sph_coord_conv
;
; PURPOSE: convert long,lat in sph system 1 to those in sph system 2
;    using rotation matrix rtot_12.
;
; CALLING SEQUENCE:
;sph_coord_conv, long1, lat1, rtot_12, long2, lat2, inverse=inverse
;        This routine is fully vectorized
;       All angles are in DEGREES
;
; INPUTS:
;RTOT_12: the rotation matrix for converting sys 1 ---> sys 2. 
;
;INPUTS/OUTPUTS:
;if (LONG1,LAT1) are given and you want (LONG2,LAT2) then
;       set INVERSE=0
;if (LONG2,LAT2) are given and you want (LONG1,LAT1) then
;       set INVERSE=1
;LONG1, LAT1: the long and lat in system 1. units are DEGREES
;LONG2, LAT2: the long and lat in system 2. units are DEGREES
;
;EXAMPLE:
; You want to define angles relative to a coord system whose pole
; lies at Galactic coords (longpole, latpole). 
;
;FIRST, get the rotation matrix rtot_12:
;  RMATRIX, longpole, latpole, rtot_12, longoffset=longoffset, $
;       noleft=noleft
;
;NEXT: you have a position in Galactic coords (ell, bee) and you want the
;      longitude and latitude (long2,lat2) in the new system:
;       SPH_COORD_CONV, ell, bee, rtot_12, long2, lat2
;OR: you have a position in system 2 (long2,lat2) and you want them in
;       Galactic coords (ell, bee):
;       SPH_COORD_CONV, ell, bee, rtot_12, long2, lat2, /INVERSE
;
; MODIFICATION HISTORY:
;6 feb 2008 by carl h.
;09dec2017: 'inverse' option defined and clarified (and fixed). Added
;documentation, particularly about NOLEFT in RMATRIX
;-

;if inverse=0, then (long_in,lat_in)=(long1, lat1)
;if inverse=1, then (long_in,lat_in)=(long2, lat2)
  
  if keyword_set(inverse) then begin
     long_in=long2
     lat_in=lat2
  endif else begin
     long_in=long1
     lat_in=lat1
  endelse
  
  ;stop

  resu = size( long_in)
xin = fltarr( 3, resu(resu( 0)+2))
xout = xin
angout = fltarr( 2, resu( 1))
xin( 0, *) = cos(!dtor * lat_in) * cos(!dtor * long_in)
xin( 1, *) = cos(!dtor * lat_in) * sin(!dtor * long_in)
xin( 2, *) = sin(!dtor * lat_in)

if keyword_set( inverse) then $
xout = transpose( rtot_12) # xin $ 
else xout= rtot_12 # xin 

long_out = reform( !radeg * atan(xout( 1, *), xout( 0, *)))
lat_out = reform( !radeg * asin(xout( 2, *)))

if n_elements( long_out) eq 1 then long_out=long_out[0]
if n_elements( lat_out) eq 1 then lat_out=lat_out[0]

;print, long_out
;print, lat_out
;stop,'2'

if keyword_set(inverse) then begin
     long1= long_out
     lat1= lat_out
  endif else begin
     long2= long_out
     lat2= lat_out
  endelse
  
 ; stop,'ret'
  
  return
end
